/*
This is part of ScreenVWM, which is based in part on the Litestep
shell source code.

Copyright (C) 2008 Jim Babcock
Copyright (C) 1997-2008 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "screenvwm.hpp"

WindowData *VWM::findWindow(const char *description)
{
	if(!description || !*description
	   || !stricmp(description, "foreground")
	   || !stricmp(description, "focused")
	  )
		return getForegroundWindow();
	
	if(!stricmp(description, "clicked"))
	{
		if(clickContext)
			return clickContext->task;
		else
			return NULL;
	}
	else if(!stricmp(description, "dragged"))
	{
		if(drag) {
			return drag->elementContext.task;
		} else {
			warn("Looked for 'dragged' task, but no drag context available.\n");
		}
	}
	else if(!stricmp(description, "dropped"))
	{
		if(droppedContext) {
			return droppedContext->task;
		} else {
			warn("Looked for 'dropped' task, but no drag/drop context available.\n");
		}
	}
	else if(!stricmp(description, "hovered"))
	{
		Point mousePos = getCursorPos();
		VWMPanel *panel = vwm->findPanel(mousePos.x, mousePos.y);
		mousePos.x -= panel->x;
		mousePos.y -= panel->y;
		LayoutLocation hovered = panel->elementAtPoint(mousePos.x, mousePos.y);
		return hovered.context.task;
	}
	
	return NULL;
}

